package io.tswf.transmutations.transmute.injection.reader

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.annotation.ReturningConstantIfTrue
import io.tswf.transmutations.transmute.annotation.ReturningValueIfNotNull
import io.tswf.transmutations.transmute.injection.returnvalue.ReturnConstantIfTransmutationTrueReturnValueProcessor
import io.tswf.transmutations.transmute.injection.returnvalue.ReturnObjectIfNotNullReturnValueProcessor
import io.tswf.transmutations.transmute.util.ConstantUtil
import org.objectweb.asm.tree.AnnotationNode

@AutoService(IReturnValueProcessorReader)
@CompileStatic
class ReturningConstantIfTrueProcessorReader implements IReturnValueProcessorReader {

    @Override
    void handleReturnValueProcessorReading(ReturnValueProcessorReadingContext readingContext) {
        def requestedConstant = readingContext.transmutationMetadata.transmutationMethod
                .findAnnotationByType(ReturningConstantIfTrue)
                ?.getValueByName("value", AnnotationNode)
                ?.with {
                    ConstantUtil.getConstantFromAnnotation(it)
                }

        if (requestedConstant != null) {
            readingContext.readedProcessors << new ReturnConstantIfTransmutationTrueReturnValueProcessor(requestedConstant)
        }
    }
}
